@@ -27,7 +27,7 @@
  * list of people on the gedit Team.
  * See the ChangeLog files for a list of changes.
  *
- * $Id: gedit-commands-file.c 6379 2008-08-10 16:40:21Z pborelli $
+ * $Id: gedit-commands-file.c 6866 2009-01-24 21:06:04Z pborelli $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -625,7 +625,6 @@ replace_read_only_file (GtkWindow *parent, GFile *file)
 	gint ret;
 	gchar *parse_name;
 	gchar *name_for_display;
-	gchar *message;
 
 	gedit_debug (DEBUG_COMMANDS);
 
@@ -638,20 +637,17 @@ replace_read_only_file (GtkWindow *parent, GFile *file)
 	name_for_display = gedit_utils_str_middle_truncate (parse_name, 50);
 	g_free (parse_name);
 
-	message = g_strdup_printf (_("The file \"%s\" is read-only."),
-				   name_for_display);
-	g_free (name_for_display);
-
 	dialog = gtk_message_dialog_new (parent,
 					 GTK_DIALOG_DESTROY_WITH_PARENT,
 					 GTK_MESSAGE_QUESTION,
 					 GTK_BUTTONS_NONE,
-					 message);
+					 _("The file \"%s\" is read-only."),
+				         name_for_display);
+	g_free (name_for_display);
 
 	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
 						  _("Do you want to try to replace it "
 						    "with the one you are saving?"));
-	g_free (message);
 
 	gtk_dialog_add_button (GTK_DIALOG (dialog),
 			       GTK_STOCK_CANCEL,
@@ -960,6 +956,20 @@ _gedit_cmd_file_save_as (GtkAction   *action,
 	file_save_as (tab, window);
 }
 
+static gboolean
+document_needs_saving (GeditDocument *doc)
+{
+	if (gtk_text_buffer_get_modified (GTK_TEXT_BUFFER (doc)))
+		return TRUE;
+
+	/* we check if it was deleted only for local files
+	 * since for remote files it may hang */
+	if (gedit_document_is_local (doc) && gedit_document_get_deleted (doc))
+		return TRUE;
+
+	return FALSE;
+}
+
 /*
  * The docs in the list must belong to the same GeditWindow.
  */
@@ -1002,9 +1012,7 @@ _gedit_cmd_file_save_documents_list (GeditWindow *window,
 			if (gedit_document_is_untitled (doc) || 
 			    gedit_document_get_readonly (doc))
 			{
-				/* Do not save unmodified utitled or readonly documents */
-				if (gtk_text_buffer_get_modified (GTK_TEXT_BUFFER (doc)) ||
-			     	    gedit_document_get_deleted (doc))
+				if (document_needs_saving (doc))
 			     	{
 				     	tabs_to_save_as = g_slist_prepend (tabs_to_save_as,
 									   t);
@@ -1228,10 +1236,10 @@ revert_dialog (GeditWindow   *window,
 					 GTK_DIALOG_DESTROY_WITH_PARENT,
 					 GTK_MESSAGE_QUESTION,
 					 GTK_BUTTONS_NONE,
-					 primary_msg);
+					 "%s", primary_msg);
 
 	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-						  secondary_msg);
+						  "%s", secondary_msg);
 	g_free (primary_msg);
 	g_free (secondary_msg);
 
@@ -1354,8 +1362,7 @@ tab_state_changed_while_saving (GeditTab    *tab,
 
 		/* If the saving operation failed or was interrupted, then the
 		   document is still "modified" -> do not close the tab */
-		if (gtk_text_buffer_get_modified (GTK_TEXT_BUFFER (doc)) ||
-		    gedit_document_get_deleted (doc))
+		if (document_needs_saving (doc))
 			return;
 
 		/* Close the document only if it has been succesfully saved.
@@ -1478,8 +1485,7 @@ save_and_close_all_documents (const GList  *docs,
 			    (state != GEDIT_TAB_STATE_REVERTING)) /* CHECK: is this the right behavior with REVERTING ?*/
 			{			
 				/* The document must be saved before closing */
-				g_return_if_fail (gtk_text_buffer_get_modified (GTK_TEXT_BUFFER (doc)) ||
-		     				  gedit_document_get_deleted (doc));
+				g_return_if_fail (document_needs_saving (doc));
 				
 				/* FIXME: manage the case of local readonly files owned by the
 				   user is running gedit - Paolo (Dec. 8, 2005) */
