@@ -27,7 +27,7 @@
  * list of people on the gedit Team.
  * See the ChangeLog files for a list of changes.
  *
- * $Id: gedit-commands-help.c,v 1.6 2006/04/20 10:40:46 sfre Exp $
+ * $Id: gedit-commands-help.c 6727 2008-12-25 21:02:48Z pborelli $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -40,10 +40,11 @@
 #include "gedit-commands.h"
 #include "gedit-debug.h"
 #include "gedit-help.h"
+#include "gedit-dirs.h"
 
 void
-gedit_cmd_help_contents (GtkAction   *action,
-			 GeditWindow *window)
+_gedit_cmd_help_contents (GtkAction   *action,
+			  GeditWindow *window)
 {
 	gedit_debug (DEBUG_COMMANDS);
 
@@ -51,12 +52,14 @@ gedit_cmd_help_contents (GtkAction   *action,
 }
 
 void
-gedit_cmd_help_about (GtkAction   *action,
-		      GeditWindow *window)
+_gedit_cmd_help_about (GtkAction   *action,
+		       GeditWindow *window)
 {
 	static const gchar * const authors[] = {
 		"Paolo Maggi <paolo@gnome.org>",
 		"Paolo Borelli <pborelli@katamail.com>",
+		"Steve Fr\303\251cinaux  <steve@istique.net>",
+		"Jesse van den Kieboom  <jesse@icecrew.nl>",
 		"James Willcox <jwillcox@gnome.org>",
 		"Chema Celorio",
 		"Federico Mena Quintero <federico@novell.com>",
@@ -79,13 +82,22 @@ gedit_cmd_help_about (GtkAction   *action,
 		   "GNOME Desktop");
 
 	GdkPixbuf *logo;
+	gchar *data_dir;
+	gchar *logo_file;
 
 	gedit_debug (DEBUG_COMMANDS);
 
-	logo = gdk_pixbuf_new_from_file (GEDIT_DATADIR "/logo/gedit-logo.png",
-					 NULL);
+	data_dir = gedit_dirs_get_gedit_data_dir ();
+	logo_file = g_build_filename (data_dir,
+				      "logo",
+				      "gedit-logo.png",
+				      NULL);
+	g_free (data_dir);
+	logo = gdk_pixbuf_new_from_file (logo_file, NULL);
+	g_free (logo_file);
 
 	gtk_show_about_dialog (GTK_WINDOW (window),
+			       "program-name", "gedit",
 			       "authors", authors,
 			       "comments", _(comments),
 			       "copyright", copyright,
@@ -94,7 +106,6 @@ gedit_cmd_help_about (GtkAction   *action,
 			       "translator-credits", _("translator-credits"),
 			       "version", VERSION,
 			       "website", "http://www.gedit.org",
-			       "name", "gedit",
 			       NULL);
 
 	if (logo)
