@@ -28,7 +28,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-commands.h 6631 2008-11-29 11:06:49Z pborelli $
+ * $Id: gedit-commands.h 6819 2009-01-07 14:56:55Z icq $
  */
 
 #ifndef __GEDIT_COMMANDS_H__
@@ -117,6 +117,10 @@ void		_gedit_cmd_view_show_side_pane		(GtkAction   *action,
 							 GeditWindow *window);
 void		_gedit_cmd_view_show_bottom_pane	(GtkAction   *action,
 							 GeditWindow *window);
+void		_gedit_cmd_view_toggle_fullscreen_mode	(GtkAction   *action,
+							 GeditWindow *window);
+void		_gedit_cmd_view_leave_fullscreen_mode	(GtkAction   *action,
+							 GeditWindow *window);
 
 void		_gedit_cmd_search_find			(GtkAction   *action,
 							 GeditWindow *window);
