@@ -20,6 +20,9 @@ typedef struct
 	guint root_changed_id;
 	guint begin_loading_id;
 	guint end_loading_id;
+
+	GList *merge_ids;
+	GtkActionGroup *merged_actions;
 	
 	GeditMessageBus *bus;
 	GeditFileBrowserWidget *widget;
@@ -41,6 +44,8 @@ window_data_new (GeditWindow            *window,
 		 GeditFileBrowserWidget *widget)
 {
 	WindowData *data = g_slice_new (WindowData);
+	GtkUIManager *manager;
+	GList *groups;
 	
 	data->bus = gedit_window_get_message_bus (window);
 	data->widget = widget;
@@ -48,11 +53,20 @@ window_data_new (GeditWindow            *window,
 						    g_str_equal,
 						    (GDestroyNotify)g_free,
 						    (GDestroyNotify)gtk_tree_row_reference_free);
+
 	data->filters = g_hash_table_new_full (g_str_hash,
 					       g_str_equal,
 					       (GDestroyNotify)g_free,
 					       NULL);
-			    
+	
+	manager = gedit_file_browser_widget_get_ui_manager (widget);
+
+	data->merge_ids = NULL;
+	data->merged_actions = gtk_action_group_new ("MessageMergedActions");
+	
+	groups = gtk_ui_manager_get_action_groups (manager);
+	gtk_ui_manager_insert_action_group (manager, data->merged_actions, g_list_length (groups));
+	
 	g_object_set_data (G_OBJECT (window), WINDOW_DATA_KEY, data);
 
 	return data;
@@ -68,10 +82,21 @@ static void
 window_data_free (GeditWindow *window)
 {
 	WindowData *data = get_window_data (window);
-	
+	GtkUIManager *manager;
+	GList *item;
+		
 	g_hash_table_destroy (data->row_tracking);	
 	g_hash_table_destroy (data->filters);
 
+	manager = gedit_file_browser_widget_get_ui_manager (data->widget);
+	gtk_ui_manager_remove_action_group (manager, data->merged_actions);
+	
+	for (item = data->merge_ids; item; item = item->next)
+		gtk_ui_manager_remove_ui (manager, GPOINTER_TO_INT (item->data));
+
+	g_list_free (data->merge_ids);
+	g_object_unref (data->merged_actions);
+
 	g_slice_free (WindowData, data);
 
 	g_object_set_data (G_OBJECT (window), WINDOW_DATA_KEY, NULL);
@@ -148,9 +173,24 @@ message_cache_data_new (GeditWindow            *window,
 }
 
 static void
-message_set_root_cb (GeditMessageBus        *bus,
-		     GeditMessage           *message,
-		     WindowData             *data)
+message_get_root_cb (GeditMessageBus *bus,
+		     GeditMessage    *message,
+		     WindowData      *data)
+{
+	GeditFileBrowserStore *store;
+	gchar *uri;
+	
+	store = gedit_file_browser_widget_get_browser_store (data->widget);
+	uri = gedit_file_browser_store_get_virtual_root (store);
+	
+	gedit_message_set (message, "uri", uri, NULL);
+	g_free (uri);
+}
+
+static void
+message_set_root_cb (GeditMessageBus *bus,
+		     GeditMessage    *message,
+		     WindowData      *data)
 {
 	gchar *root = NULL;
 	gchar *virtual = NULL;
@@ -513,6 +553,78 @@ message_show_files_cb (GeditMessageBus *bus,
 	gedit_file_browser_widget_show_files (data->widget);
 }
 
+static void
+message_add_context_item_cb (GeditMessageBus *bus,
+			     GeditMessage    *message,
+			     WindowData      *data)
+{
+	GtkAction *action = NULL;
+	gchar *path = NULL;
+	gchar *name;
+	GtkUIManager *manager;
+	guint merge_id;
+	
+	gedit_message_get (message, 
+			   "action", &action, 
+			   "path", &path, 
+			   NULL);
+	
+	if (!action || !path)
+	{
+		if (action)
+			g_object_unref (action);
+
+		g_free (path);
+		return;
+	}
+	
+	gtk_action_group_add_action (data->merged_actions, action);
+	manager = gedit_file_browser_widget_get_ui_manager (data->widget);
+	name = g_strconcat (gtk_action_get_name (action), "MenuItem", NULL);
+	merge_id = gtk_ui_manager_new_merge_id (manager);
+	
+	gtk_ui_manager_add_ui (manager, 
+			       merge_id,
+			       path,
+			       name,
+			       gtk_action_get_name (action),
+			       GTK_UI_MANAGER_AUTO,
+			       FALSE);
+	
+	if (gtk_ui_manager_get_widget (manager, path))
+	{
+		data->merge_ids = g_list_prepend (data->merge_ids, GINT_TO_POINTER (merge_id));
+		gedit_message_set (message, "id", merge_id, NULL);
+	}
+	else
+	{
+		gedit_message_set (message, "id", 0, NULL);
+	}
+	
+	g_object_unref (action);
+	g_free (path);
+	g_free (name);
+}
+
+static void
+message_remove_context_item_cb (GeditMessageBus *bus,
+				GeditMessage    *message,
+				WindowData      *data)
+{
+	guint merge_id = 0;
+	GtkUIManager *manager;
+	
+	gedit_message_get (message, "id", &merge_id, NULL);
+	
+	if (merge_id == 0)
+		return;
+	
+	manager = gedit_file_browser_widget_get_ui_manager (data->widget);
+		
+	data->merge_ids = g_list_remove (data->merge_ids, GINT_TO_POINTER (merge_id));
+	gtk_ui_manager_remove_ui (manager, merge_id);
+}
+
 static void
 register_methods (GeditWindow            *window,
 		  GeditFileBrowserWidget *widget)
@@ -521,6 +633,12 @@ register_methods (GeditWindow            *window,
 	WindowData *data = get_window_data (window);
 
 	/* Register method calls */
+	gedit_message_bus_register (bus, 
+				    MESSAGE_OBJECT_PATH, "get_root", 
+				    1,
+				    "uri", G_TYPE_STRING,
+				    NULL);
+ 
 	gedit_message_bus_register (bus, 
 				    MESSAGE_OBJECT_PATH, "set_root", 
 				    1, 
@@ -549,6 +667,19 @@ register_methods (GeditWindow            *window,
 				    "id", G_TYPE_ULONG,
 				    NULL);
 	
+	gedit_message_bus_register (bus,
+				    MESSAGE_OBJECT_PATH, "add_context_item",
+				    1,
+    				    "action", GTK_TYPE_ACTION,
+    				    "id", G_TYPE_UINT,
+				    NULL);
+
+	gedit_message_bus_register (bus,
+				    MESSAGE_OBJECT_PATH, "remove_context_item",
+				    0,
+				    "id", G_TYPE_UINT,
+				    NULL);
+	
 	gedit_message_bus_register (bus, MESSAGE_OBJECT_PATH, "up", 0, NULL);
 	
 	gedit_message_bus_register (bus, MESSAGE_OBJECT_PATH, "history_back", 0, NULL);
@@ -570,11 +701,15 @@ register_methods (GeditWindow            *window,
 	gedit_message_bus_register (bus, MESSAGE_OBJECT_PATH, "show_bookmarks", 0, NULL);
 	gedit_message_bus_register (bus, MESSAGE_OBJECT_PATH, "show_files", 0, NULL);
 
+	BUS_CONNECT (bus, get_root, data);
 	BUS_CONNECT (bus, set_root, data);
 	BUS_CONNECT (bus, set_emblem, data);
 	BUS_CONNECT (bus, add_filter, window);
 	BUS_CONNECT (bus, remove_filter, data);
 
+	BUS_CONNECT (bus, add_context_item, data);
+	BUS_CONNECT (bus, remove_context_item, data);
+
 	BUS_CONNECT (bus, up, data);
 	BUS_CONNECT (bus, history_back, data);
 	BUS_CONNECT (bus, history_forward, data);
