@@ -14,16 +14,19 @@
 #    You should have received a copy of the GNU General Public License
 #    along with this program; if not, write to the Free Software
 #    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-    
-import gedit
-import gtk
-from gtk import gdk
+
 import sys
 import os
 import shutil
-from SnippetPluginInstance import SnippetsPluginInstance
-from SnippetsLibrary import SnippetsLibrary
-from SnippetsDialog import SnippetsDialog
+
+import gtk
+from gtk import gdk    
+import gedit
+import platform
+
+from WindowHelper import WindowHelper
+from Library import Library
+from Manager import Manager
 from Snippet import Snippet
 
 class SnippetsPlugin(gedit.Plugin):
@@ -32,30 +35,36 @@ class SnippetsPlugin(gedit.Plugin):
 
                 self.dlg = None
                 
-                library = SnippetsLibrary()
+                library = Library()
                 library.set_accelerator_callback(self.accelerator_activated)
                 
-                userdir = os.path.join(os.environ['HOME'], '.gnome2', 'gedit', 'snippets')
+                if platform.platform() == 'Windows':
+	                userdir = os.path.expanduser('~/gedit/snippets')
+	        else:
+	        	userdir = os.path.expanduser('~/.gnome2/gedit/snippets')
+
                 library.set_dirs(userdir, self.system_dirs())
         
         def system_dirs(self):
-                if 'XDG_DATA_DIRS' in os.environ:
-                        datadirs = os.environ['XDG_DATA_DIRS']
-                else:
-                        datadirs = '/usr/local/share:/usr/share'
-                
-                dirs = []
-                
-                for d in datadirs.split(':'):
-                        d = os.path.join(d, 'gedit-2', 'plugins', 'snippets')
-                        
-                        if os.path.isdir(d):
-                                dirs.append(d)
+        	if platform.platform() != 'Windows':
+		        if 'XDG_DATA_DIRS' in os.environ:
+		                datadirs = os.environ['XDG_DATA_DIRS']
+		        else:
+		                datadirs = '/usr/local/share:/usr/share'
+		        
+		        dirs = []
+		        
+		        for d in datadirs.split(':'):
+		                d = os.path.join(d, 'gedit-2', 'plugins', 'snippets')
+		                
+		                if os.path.isdir(d):
+		                        dirs.append(d)
                 
+                dirs.append(self.get_data_dir())
                 return dirs
         
         def activate(self, window):
-                data = SnippetsPluginInstance(self)
+                data = WindowHelper(self)
                 window._snippets_plugin_data = data
                 data.run(window)
 
@@ -68,7 +77,7 @@ class SnippetsPlugin(gedit.Plugin):
         
         def create_configure_dialog(self):
                 if not self.dlg:
-                        self.dlg = SnippetsDialog()
+                        self.dlg = Manager(self.get_data_dir())
                 else:
                         self.dlg.run()
                 
